@@ -1,5 +1,5 @@
-#from dirg_util.dict import LDAPDict
-#ldap_settings = {
+# from dirg_util.dict import LDAPDict
+# ldap_settings = {
 #    "ldapuri": "ldaps://ldap.test.umu.se",
 #    "base": "dc=umu, dc=se",
 #    "filter_pattern": "(uid=%s)",
@@ -30,9 +30,9 @@
 #    "exact_match": True,
 #    "firstonly_len1": True,
 #    "timeout": 15,
-#}
-#Uncomment to use a LDAP directory instead.
-#USERS = LDAPDict(**ldap_settings)
+# }
+# Uncomment to use a LDAP directory instead.
+# USERS = LDAPDict(**ldap_settings)
 
 USERS = {
     "testuser": {
@@ -47,11 +47,16 @@ USERS = {
         "o": "Example Co.",
         "ou": "IT",
         "initials": "P",
+        "co": "co",
+        "mail": "mail",
+        "noreduorgacronym": "noreduorgacronym",
         "schacHomeOrganization": "example.com",
         "email": "test@example.com",
         "displayName": "Test Testsson",
         "labeledURL": "http://www.example.com/test My homepage",
-        "norEduPersonNIN": "SE199012315555"
+        "norEduPersonNIN": "SE199012315555",
+        "postaladdress": "postaladdress",
+        "cn": "cn"
     },
     "roland": {
         "sn": "Hedberg",
@@ -64,8 +69,8 @@ USERS = {
         "o": "Example Co.",
         "ou": "IT",
         "initials": "P",
-        #"schacHomeOrganization": "example.com",
-        "email": "roland@example.com",
+        # "schacHomeOrganization": "example.com",
+        "mail": "roland@example.com",
         "displayName": "P. Roland Hedberg",
         "labeledURL": "http://www.example.com/rohe My homepage",
         "norEduPersonNIN": "SE197001012222"
@@ -88,4 +93,4 @@ EXTRA = {
         "schacGender": "male",
         "schacUserPresenceID": "skype:pepe.perez"
     }
-}
\ No newline at end of file
+}
