@@ -1,10 +1,6 @@
 import logging
-from urllib import urlencode
-from urlparse import parse_qs
-from urlparse import urlsplit
 import six
 import time
-import ldap
 from saml2 import SAMLError
 from saml2.aes import AESCipher
 from saml2.httputil import Response
@@ -13,6 +9,8 @@ from saml2.httputil import Redirect
 from saml2.httputil import Unauthorized
 from saml2.httputil import parse_cookie
 
+from six.moves.urllib.parse import urlencode, parse_qs, urlsplit
+
 __author__ = 'rolandh'
 
 logger = logging.getLogger(__name__)
@@ -148,7 +146,8 @@ class UsernamePasswordMako(UserAuthnMethod):
         return resp
 
     def _verify(self, pwd, user):
-        assert is_equal(pwd, self.passwd[user])
+        if not is_equal(pwd, self.passwd[user]):
+            raise ValueError("Wrong password")
 
     def verify(self, request, **kwargs):
         """
@@ -178,7 +177,7 @@ class UsernamePasswordMako(UserAuthnMethod):
             return_to = create_return_url(self.return_to, _dict["query"][0],
                                           **{self.query_param: "true"})
             resp = Redirect(return_to, headers=[cookie])
-        except (AssertionError, KeyError):
+        except (ValueError, KeyError):
             resp = Unauthorized("Unknown user or wrong password")
 
         return resp
@@ -232,33 +231,38 @@ class AuthnMethodChooser(object):
         else:
             pass  # TODO
 
+try:
+    import ldap
 
-class LDAPAuthn(UsernamePasswordMako):
-    def __init__(self, srv, ldapsrv, return_to,
-                 dn_pattern, mako_template, template_lookup):
-        """
-        :param srv: The server instance
-        :param ldapsrv: Which LDAP server to us
-        :param return_to: Where to send the user after authentication
-        :return:
-        """
-        UsernamePasswordMako.__init__(self, srv, mako_template, template_lookup,
-                                      None, return_to)
+    class LDAPAuthn(UsernamePasswordMako):
+        def __init__(self, srv, ldapsrv, return_to,
+                     dn_pattern, mako_template, template_lookup):
+            """
+            :param srv: The server instance
+            :param ldapsrv: Which LDAP server to us
+            :param return_to: Where to send the user after authentication
+            :return:
+            """
+            UsernamePasswordMako.__init__(self, srv, mako_template, template_lookup,
+                                          None, return_to)
 
-        self.ldap = ldap.initialize(ldapsrv)
-        self.ldap.protocol_version = 3
-        self.ldap.set_option(ldap.OPT_REFERRALS, 0)
-        self.dn_pattern = dn_pattern
+            self.ldap = ldap.initialize(ldapsrv)
+            self.ldap.protocol_version = 3
+            self.ldap.set_option(ldap.OPT_REFERRALS, 0)
+            self.dn_pattern = dn_pattern
 
-    def _verify(self, pwd, user):
-        """
-        Verifies the username and password agains a LDAP server
-        :param pwd: The password
-        :param user: The username
-        :return: AssertionError if the LDAP verification failed.
-        """
-        _dn = self.dn_pattern % user
-        try:
-            self.ldap.simple_bind_s(_dn, pwd)
-        except Exception:
-            raise AssertionError()
+        def _verify(self, pwd, user):
+            """
+            Verifies the username and password agains a LDAP server
+            :param pwd: The password
+            :param user: The username
+            :return: AssertionError if the LDAP verification failed.
+            """
+            _dn = self.dn_pattern % user
+            try:
+                self.ldap.simple_bind_s(_dn, pwd)
+            except Exception:
+                raise AssertionError()
+except ImportError:
+    class LDAPAuthn(UserAuthnMethod):
+        pass
