@@ -15,18 +15,6 @@ from saml2 import xmldsig
 from saml2 import xmlenc
 
 
-ONTS = {
-    saml.NAMESPACE: saml,
-    mdui.NAMESPACE: mdui,
-    mdattr.NAMESPACE: mdattr,
-    dri.NAMESPACE: dri,
-    ui.NAMESPACE: ui,
-    idpdisc.NAMESPACE: idpdisc,
-    md.NAMESPACE: md,
-    xmldsig.NAMESPACE: xmldsig,
-    xmlenc.NAMESPACE: xmlenc
-}
-
 __author__ = 'rolandh'
 
 logger = logging.getLogger(__name__)
@@ -72,7 +60,7 @@ class SessionStorage(object):
         try:
             statements = self.authn[key]
         except KeyError:
-            logger.info("Unknown subject %s" % name_id)
+            logger.info("Unknown subject %s", name_id)
             return []
 
         for statement in statements:
@@ -88,7 +76,7 @@ class SessionStorage(object):
         return result
 
     def remove_authn_statements(self, name_id):
-        logger.debug("remove authn about: %s" % name_id)
+        logger.debug("remove authn about: %s", name_id)
         nkey = sha1(code_binary(name_id)).hexdigest()
 
         del self.authn[nkey]
