@@ -8,26 +8,24 @@ Based on the use of xmlsec1 binaries and not the python xmlsec module.
 from OpenSSL import crypto
 
 import base64
-from base64 import b64decode
 import hashlib
 import logging
 import os
 import ssl
-from six.moves.urllib.parse import urlencode
+import six
 
 from time import mktime
 from binascii import hexlify
-import six
 
-from Crypto.PublicKey.RSA import importKey
-from Crypto.Signature import PKCS1_v1_5
-from Crypto.Util.asn1 import DerSequence
-from Crypto.PublicKey import RSA
-from Crypto.Hash import SHA
-from Crypto.Hash import SHA224
-from Crypto.Hash import SHA256
-from Crypto.Hash import SHA384
-from Crypto.Hash import SHA512
+from future.backports.urllib.parse import urlencode
+
+from cryptography.exceptions import InvalidSignature
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives.asymmetric import rsa
+from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15
+from cryptography.hazmat.primitives.serialization import load_pem_private_key
+from cryptography.x509 import load_pem_x509_certificate
 
 from tempfile import NamedTemporaryFile
 from subprocess import Popen
@@ -77,6 +75,8 @@ XMLTAG = "<?xml version='1.0'?>"
 PREFIX1 = "<?xml version='1.0' encoding='UTF-8'?>"
 PREFIX2 = '<?xml version="1.0" encoding="UTF-8"?>'
 
+backend = default_backend()
+
 
 class SigverError(SAMLError):
     pass
@@ -144,6 +144,12 @@ def rm_xmltag(statement):
 
 
 def signed(item):
+    """
+    Is any part of the document signed ?
+
+    :param item: A Samlbase instance
+    :return: True if some part of it is signed
+    """
     if SIG in item.c_children.keys() and item.signature:
         return True
     else:
@@ -236,7 +242,7 @@ def _make_vals(val, klass, seccont, klass_inst=None, prop=None, part=False,
     """
     cinst = None
 
-    #print("make_vals(%s, %s)" % (val, klass))
+    # print("make_vals(%s, %s)" % (val, klass))
 
     if isinstance(val, dict):
         cinst = _instance(klass, val, seccont, base64encode=base64encode,
@@ -265,7 +271,7 @@ def _instance(klass, ava, seccont, base64encode=False, elements_to_sign=None):
     instance = klass()
 
     for prop in instance.c_attributes.values():
-        #print("# %s" % (prop))
+        # print("# %s" % (prop))
         if prop in ava:
             if isinstance(ava[prop], bool):
                 setattr(instance, prop, str(ava[prop]).encode('utf-8'))
@@ -278,9 +284,9 @@ def _instance(klass, ava, seccont, base64encode=False, elements_to_sign=None):
         instance.set_text(ava["text"], base64encode)
 
     for prop, klassdef in instance.c_children.values():
-        #print("## %s, %s" % (prop, klassdef))
+        # print("## %s, %s" % (prop, klassdef))
         if prop in ava:
-            #print("### %s" % ava[prop])
+            # print("### %s" % ava[prop])
             if isinstance(klassdef, list):
                 # means there can be a list of values
                 _make_vals(ava[prop], klassdef[0], seccont, instance, prop,
@@ -320,9 +326,9 @@ def signed_instance_factory(instance, seccont, elements_to_sign=None):
             signed_xml = seccont.sign_statement(
                 signed_xml, node_name=node_name, node_id=nodeid)
 
-        #print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx")
-        #print("%s" % signed_xml)
-        #print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx")
+        # print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx")
+        # print("%s" % signed_xml)
+        # print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx")
         return signed_xml
     else:
         return instance
@@ -353,7 +359,10 @@ def make_temp(string, suffix="", decode=True, delete=True):
         xmlsec function).
     """
     ntf = NamedTemporaryFile(suffix=suffix, delete=delete)
-    assert isinstance(string, six.binary_type)
+    # Python3 tempfile requires byte-like object
+    if not isinstance(string, six.binary_type):
+        string = string.encode("utf8")
+
     if decode:
         ntf.write(base64.b64decode(string))
     else:
@@ -365,6 +374,7 @@ def make_temp(string, suffix="", decode=True, delete=True):
 def split_len(seq, length):
     return [seq[i:i + length] for i in range(0, len(seq), length)]
 
+
 # --------------------------------------------------------------------------
 
 M2_TIME_FORMAT = "%b %d %H:%M:%S %Y"
@@ -386,23 +396,16 @@ def active_cert(key):
     """
     try:
         cert_str = pem_format(key)
-        try:
-            certificate = importKey(cert_str)
-            not_before = to_time(str(certificate.get_not_before()))
-            not_after = to_time(str(certificate.get_not_after()))
-            assert not_before < utc_now()
-            assert not_after > utc_now()
-            return True
-        except:
-                cert = crypto.load_certificate(crypto.FILETYPE_PEM, cert_str)
-                assert cert.has_expired() == 0
-                assert not OpenSSLWrapper().certificate_not_valid_yet(cert)
-                return True
+        cert = crypto.load_certificate(crypto.FILETYPE_PEM, cert_str)
+        assert cert.has_expired() == 0
+        assert not OpenSSLWrapper().certificate_not_valid_yet(cert)
+        return True
     except AssertionError:
         return False
     except AttributeError:
         return False
 
+
 def cert_from_key_info(key_info, ignore_age=False):
     """ Get all X509 certs from a KeyInfo instance. Care is taken to make sure
     that the certs are continues sequences of bytes.
@@ -416,7 +419,7 @@ def cert_from_key_info(key_info, ignore_age=False):
     """
     res = []
     for x509_data in key_info.x509_data:
-        #print("X509Data",x509_data)
+        # print("X509Data",x509_data)
         x509_certificate = x509_data.x509_certificate
         cert = x509_certificate.text.strip()
         cert = "\n".join(split_len("".join([s.strip() for s in
@@ -513,13 +516,13 @@ def key_from_key_value_dict(key_info):
 # =============================================================================
 
 
-#def rsa_load(filename):
+# def rsa_load(filename):
 #    """Read a PEM-encoded RSA key pair from a file."""
 #    return M2Crypto.RSA.load_key(filename, M2Crypto.util
 # .no_passphrase_callback)
 #
 #
-#def rsa_loads(key):
+# def rsa_loads(key):
 #    """Read a PEM-encoded RSA key pair from a string."""
 #    return M2Crypto.RSA.load_key_string(key,
 #                                        M2Crypto.util.no_passphrase_callback)
@@ -534,19 +537,8 @@ def rsa_eq(key1, key2):
 
 
 def extract_rsa_key_from_x509_cert(pem):
-    # Convert from PEM to DER
-    der = ssl.PEM_cert_to_DER_cert(pem.decode('ascii'))
-
-    # Extract subjectPublicKeyInfo field from X.509 certificate (see RFC3280)
-    cert = DerSequence()
-    cert.decode(der)
-    tbsCertificate = DerSequence()
-    tbsCertificate.decode(cert[0])
-    subjectPublicKeyInfo = tbsCertificate[6]
-
-    # Initialize RSA key
-    rsa_key = RSA.importKey(subjectPublicKeyInfo)
-    return rsa_key
+    cert = load_pem_x509_certificate(pem, backend)
+    return cert.public_key()
 
 
 def pem_format(key):
@@ -555,7 +547,7 @@ def pem_format(key):
 
 
 def import_rsa_key_from_file(filename):
-    return RSA.importKey(read_file(filename, 'r'))
+    return load_pem_private_key(read_file(filename, 'rb'), None, backend)
 
 
 def parse_xmlsec_output(output):
@@ -565,7 +557,7 @@ def parse_xmlsec_output(output):
     :param output: The output from Popen
     :return: A boolean; True if the command was a success otherwise False
     """
-    for line in output.split("\n"):
+    for line in output.splitlines():
         if line == "OK":
             return True
         elif line == "FAIL":
@@ -580,6 +572,9 @@ def sha1_digest(msg):
 class Signer(object):
     """Abstract base class for signing algorithms."""
 
+    def __init__(self, key):
+        self.key = key
+
     def sign(self, msg, key):
         """Sign ``msg`` with ``key`` and return the signature."""
         raise NotImplementedError
@@ -590,33 +585,61 @@ class Signer(object):
 
 
 class RSASigner(Signer):
-    def __init__(self, digest):
+    def __init__(self, digest, key=None):
+        Signer.__init__(self, key)
         self.digest = digest
 
-    def sign(self, msg, key):
-        h = self.digest.new(msg)
-        signer = PKCS1_v1_5.new(key)
-        return signer.sign(h)
+    def sign(self, msg, key=None):
+        if key is None:
+            key = self.key
 
-    def verify(self, msg, sig, key):
-        h = self.digest.new(msg)
-        verifier = PKCS1_v1_5.new(key)
-        return verifier.verify(h, sig)
+        return key.sign(msg, PKCS1v15(), self.digest)
+
+    def verify(self, msg, sig, key=None):
+        if key is None:
+            key = self.key
+
+        try:
+            if isinstance(key, rsa.RSAPrivateKey):
+                key = key.public_key()
+
+            key.verify(sig, msg, PKCS1v15(), self.digest)
+            return True
+        except InvalidSignature:
+            return False
 
 
 SIGNER_ALGS = {
-    SIG_RSA_SHA1: RSASigner(SHA),
-    SIG_RSA_SHA224: RSASigner(SHA224),
-    SIG_RSA_SHA256: RSASigner(SHA256),
-    SIG_RSA_SHA384: RSASigner(SHA384),
-    SIG_RSA_SHA512: RSASigner(SHA512),
+    SIG_RSA_SHA1: RSASigner(hashes.SHA1()),
+    SIG_RSA_SHA224: RSASigner(hashes.SHA224()),
+    SIG_RSA_SHA256: RSASigner(hashes.SHA256()),
+    SIG_RSA_SHA384: RSASigner(hashes.SHA384()),
+    SIG_RSA_SHA512: RSASigner(hashes.SHA512()),
 }
 
 REQ_ORDER = ["SAMLRequest", "RelayState", "SigAlg"]
 RESP_ORDER = ["SAMLResponse", "RelayState", "SigAlg"]
 
 
-def verify_redirect_signature(saml_msg, cert=None, sigkey=None):
+class RSACrypto(object):
+    def __init__(self, key):
+        self.key = key
+
+    def get_signer(self, sigalg, sigkey=None):
+        try:
+            signer = SIGNER_ALGS[sigalg]
+        except KeyError:
+            return None
+        else:
+            if sigkey:
+                signer.key = sigkey
+            else:
+                signer.key = self.key
+
+        return signer
+
+
+def verify_redirect_signature(saml_msg, crypto, cert=None, sigkey=None):
     """
 
     :param saml_msg: A dictionary with strings as values, *NOT* lists as
@@ -626,7 +649,7 @@ def verify_redirect_signature(saml_msg, cert=None, sigkey=None):
     """
 
     try:
-        signer = SIGNER_ALGS[saml_msg["SigAlg"]]
+        signer = crypto.get_signer(saml_msg["SigAlg"], sigkey)
     except KeyError:
         raise Unsupported("Signature algorithm: %s" % saml_msg["SigAlg"])
     else:
@@ -644,10 +667,12 @@ def verify_redirect_signature(saml_msg, cert=None, sigkey=None):
             string = "&".join(
                 [urlencode({k: _args[k]}) for k in _order if k in
                  _args]).encode('ascii')
+
             if cert:
                 _key = extract_rsa_key_from_x509_cert(pem_format(cert))
             else:
                 _key = sigkey
+
             _sign = base64.b64decode(saml_msg["Signature"])
 
             return bool(signer.verify(string, _sign, _key))
@@ -657,6 +682,13 @@ LOG_LINE = 60 * "=" + "\n%s\n" + 60 * "-" + "\n%s" + 60 * "="
 LOG_LINE_2 = 60 * "=" + "\n%s\n%s\n" + 60 * "-" + "\n%s" + 60 * "="
 
 
+def make_str(txt):
+    if isinstance(txt, six.string_types):
+        return txt
+    else:
+        return txt.decode("utf8")
+
+
 # ---------------------------------------------------------------------------
 
 
@@ -669,34 +701,35 @@ def read_cert_from_file(cert_file, cert_type):
     :return: A base64 encoded certificate as a string or the empty string
     """
 
-
     if not cert_file:
         return ""
 
     if cert_type == "pem":
-        line = open(cert_file).read().replace("\r\n", "\n").split("\n")
+        _a = read_file(cert_file, 'rb').decode("utf8")
+        _b = _a.replace("\r\n", "\n")
+        lines = _b.split("\n")
 
-        if line[0] == "-----BEGIN CERTIFICATE-----":
-            line = line[1:]
-        elif line[0] == "-----BEGIN PUBLIC KEY-----":
-            line = line[1:]
+        for pattern in ("-----BEGIN CERTIFICATE-----",
+                        "-----BEGIN PUBLIC KEY-----"):
+            if pattern in lines:
+                lines = lines[lines.index(pattern) + 1:]
+                break
         else:
             raise CertificateError("Strange beginning of PEM file")
 
-        while line[-1] == "":
-            line = line[:-1]
-
-        if line[-1] == "-----END CERTIFICATE-----":
-            line = line[:-1]
-        elif line[-1] == "-----END PUBLIC KEY-----":
-            line = line[:-1]
+        for pattern in ("-----END CERTIFICATE-----",
+                        "-----END PUBLIC KEY-----"):
+            if pattern in lines:
+                lines = lines[:lines.index(pattern)]
+                break
         else:
             raise CertificateError("Strange end of PEM file")
-        return "".join(line)
+        return make_str("".join(lines).encode("utf8"))
 
     if cert_type in ["der", "cer", "crt"]:
-        data = read_file(cert_file)
-        return base64.b64encode(str(data))
+        data = read_file(cert_file, 'rb')
+        _cert = base64.b64encode(data)
+        return make_str(_cert)
 
 
 class CryptoBackend():
@@ -746,10 +779,16 @@ class CryptoBackendXmlSec1(CryptoBackend):
         else:
             self._xmlsec_delete_tmpfiles = True
 
+        try:
+            self.non_xml_crypto = RSACrypto(kwargs['rsa_key'])
+        except KeyError:
+            pass
+
     def version(self):
         com_list = [self.xmlsec, "--version"]
         pof = Popen(com_list, stderr=PIPE, stdout=PIPE)
         content = pof.stdout.read().decode('ascii')
+        pof.wait()
         try:
             return content.split(" ")[1]
         except IndexError:
@@ -766,7 +805,7 @@ class CryptoBackendXmlSec1(CryptoBackend):
         :param xpath: What should be encrypted
         :return:
         """
-        logger.debug("Encryption input len: %d" % len(text))
+        logger.debug("Encryption input len: %d", len(text))
         _, fil = make_temp(str(text).encode('utf-8'), decode=False)
 
         com_list = [self.xmlsec, "--encrypt", "--pubkey-cert-pem", recv_key,
@@ -797,7 +836,8 @@ class CryptoBackendXmlSec1(CryptoBackend):
         if isinstance(statement, SamlBase):
             statement = pre_encrypt_assertion(statement)
 
-        _, fil = make_temp(str(statement).encode('utf-8'), decode=False, delete=False)
+        _, fil = make_temp(str(statement).encode('utf-8'), decode=False,
+                           delete=False)
         _, tmpl = make_temp(str(template).encode('utf-8'), decode=False)
 
         if not node_xpath:
@@ -826,7 +866,7 @@ class CryptoBackendXmlSec1(CryptoBackend):
         :return: The decrypted document
         """
 
-        logger.debug("Decrypt input len: %d" % len(enctext))
+        logger.debug("Decrypt input len: %d", len(enctext))
         _, fil = make_temp(str(enctext).encode('utf-8'), decode=False)
 
         com_list = [self.xmlsec, "--decrypt", "--privkey-pem",
@@ -850,8 +890,8 @@ class CryptoBackendXmlSec1(CryptoBackend):
             'id','Id' or 'ID'
         :return: The signed statement
         """
-        if not isinstance(statement, six.binary_type):
-            statement = str(statement).encode('utf-8')
+        if isinstance(statement, SamlBase):
+            statement = str(statement)
 
         _, fil = make_temp(statement, suffix=".xml",
                            decode=False, delete=self._xmlsec_delete_tmpfiles)
@@ -870,8 +910,8 @@ class CryptoBackendXmlSec1(CryptoBackend):
                 if signed_statement:
                     return signed_statement.decode('utf-8')
             logger.error(
-                "Signing operation failed :\nstdout : %s\nstderr : %s" % (
-                    stdout, stderr))
+                "Signing operation failed :\nstdout : %s\nstderr : %s",
+                stdout, stderr)
             raise SigverError(stderr)
         except DecryptError:
             raise SigverError("Signing failed")
@@ -906,7 +946,7 @@ class CryptoBackendXmlSec1(CryptoBackend):
 
         if self.__DEBUG:
             try:
-                print( " ".join(com_list))
+                print(" ".join(com_list))
             except TypeError:
                 print("cert_type", cert_type)
                 print("cert_file", cert_file)
@@ -936,22 +976,23 @@ class CryptoBackendXmlSec1(CryptoBackend):
         com_list.extend(["--output", ntf.name])
         com_list += extra_args
 
-        logger.debug("xmlsec command: %s" % " ".join(com_list))
+        logger.debug("xmlsec command: %s", " ".join(com_list))
 
         pof = Popen(com_list, stderr=PIPE, stdout=PIPE)
 
         p_out = pof.stdout.read().decode('utf-8')
         p_err = pof.stderr.read().decode('utf-8')
+        pof.wait()
 
         if pof.returncode is not None and pof.returncode < 0:
-            logger.error(LOG_LINE % (p_out, p_err))
+            logger.error(LOG_LINE, p_out, p_err)
             raise XmlsecError("%d:%s" % (pof.returncode, p_err))
 
         try:
             if validate_output:
                 parse_xmlsec_output(p_err)
         except XmlsecError as exc:
-            logger.error(LOG_LINE_2 % (p_out, p_err, exc))
+            logger.error(LOG_LINE_2, p_out, p_err, exc)
             raise
 
         ntf.seek(0)
@@ -1029,7 +1070,7 @@ class CryptoBackendXMLSecurity(CryptoBackend):
 def security_context(conf, debug=None):
     """ Creates a security context based on the configuration
 
-    :param conf: The configuration
+    :param conf: The configuration, this is a Config instance
     :return: A SecurityContext instance
     """
     if not conf:
@@ -1050,6 +1091,8 @@ def security_context(conf, debug=None):
     if _only_md is None:
         _only_md = False
 
+    sec_backend = None
+
     if conf.crypto_backend == 'xmlsec1':
         xmlsec_binary = conf.xmlsec_binary
         if not xmlsec_binary:
@@ -1060,10 +1103,20 @@ def security_context(conf, debug=None):
             xmlsec_binary = get_xmlsec_binary(_path)
             # verify that xmlsec is where it's supposed to be
         if not os.path.exists(xmlsec_binary):
-            #if not os.access(, os.F_OK):
+            # if not os.access(, os.F_OK):
             raise SigverError(
                 "xmlsec binary not in '%s' !" % xmlsec_binary)
         crypto = _get_xmlsec_cryptobackend(xmlsec_binary, debug=debug)
+        _file_name = conf.getattr("key_file", "")
+        if _file_name:
+            try:
+                rsa_key = import_rsa_key_from_file(_file_name)
+            except Exception as err:
+                logger.error("Could not import key from {}: {}".format(_file_name,
+                                                                       err))
+                raise
+            else:
+                sec_backend = RSACrypto(rsa_key)
     elif conf.crypto_backend == 'XMLSecurity':
         # new and somewhat untested pyXMLSecurity crypto backend.
         crypto = CryptoBackendXMLSecurity(debug=debug)
@@ -1071,6 +1124,7 @@ def security_context(conf, debug=None):
         raise SigverError('Unknown crypto_backend %s' % (
             repr(conf.crypto_backend)))
 
+
     enc_key_files = []
     if conf.encryption_keypairs is not None:
         for _encryption_keypair in conf.encryption_keypairs:
@@ -1085,36 +1139,43 @@ def security_context(conf, debug=None):
         tmp_cert_file=conf.tmp_cert_file,
         tmp_key_file=conf.tmp_key_file,
         validate_certificate=conf.validate_certificate,
-        enc_key_files=enc_key_files, encryption_keypairs=conf.encryption_keypairs)
+        enc_key_files=enc_key_files,
+        encryption_keypairs=conf.encryption_keypairs,
+        sec_backend=sec_backend)
 
 
 def encrypt_cert_from_item(item):
     _encrypt_cert = None
     try:
         try:
-            _elem = extension_elements_to_elements(item.extensions.extension_elements,[pefim, ds])
+            _elem = extension_elements_to_elements(
+                item.extensions.extension_elements, [pefim, ds])
         except:
-            _elem = extension_elements_to_elements(item.extension_elements[0].children,
-                                                   [pefim, ds])
+            _elem = extension_elements_to_elements(
+                item.extension_elements[0].children,
+                [pefim, ds])
 
         for _tmp_elem in _elem:
             if isinstance(_tmp_elem, SPCertEnc):
                 for _tmp_key_info in _tmp_elem.key_info:
-                    if _tmp_key_info.x509_data is not None and len(_tmp_key_info.x509_data) > 0:
-                        _encrypt_cert = _tmp_key_info.x509_data[0].x509_certificate.text
+                    if _tmp_key_info.x509_data is not None and len(
+                            _tmp_key_info.x509_data) > 0:
+                        _encrypt_cert = _tmp_key_info.x509_data[
+                            0].x509_certificate.text
                         break
-            #_encrypt_cert = _elem[0].x509_data[0].x509_certificate.text
-#        else:
-#            certs = cert_from_instance(item)
-#            if len(certs) > 0:
-#                _encrypt_cert = certs[0]
+                        # _encrypt_cert = _elem[0].x509_data[
+                        # 0].x509_certificate.text
+                    #        else:
+                    #            certs = cert_from_instance(item)
+                    #            if len(certs) > 0:
+                    #                _encrypt_cert = certs[0]
     except Exception as _exception:
         pass
 
-#    if _encrypt_cert is None:
-#        certs = cert_from_instance(item)
-#        if len(certs) > 0:
-#            _encrypt_cert = certs[0]
+    #    if _encrypt_cert is None:
+    #        certs = cert_from_instance(item)
+    #        if len(certs) > 0:
+    #            _encrypt_cert = certs[0]
 
     if _encrypt_cert is not None:
         if _encrypt_cert.find("-----BEGIN CERTIFICATE-----\n") == -1:
@@ -1124,7 +1185,6 @@ def encrypt_cert_from_item(item):
     return _encrypt_cert
 
 
-
 class CertHandlerExtra(object):
     def __init__(self):
         pass
@@ -1135,14 +1195,14 @@ class CertHandlerExtra(object):
     def generate_cert(self, generate_cert_info, root_cert_string,
                       root_key_string):
         raise Exception("generate_cert function must be implemented")
-        #Excepts to return (cert_string, key_string)
+        # Excepts to return (cert_string, key_string)
 
     def use_validate_cert_func(self):
         raise Exception("use_validate_cert_func function must be implemented")
 
     def validate_cert(self, cert_str, root_cert_string, root_key_string):
         raise Exception("validate_cert function must be implemented")
-        #Excepts to return True/False
+        # Excepts to return True/False
 
 
 class CertHandler(object):
@@ -1169,7 +1229,7 @@ class CertHandler(object):
 
         self._verify_cert = False
         self._generate_cert = False
-        #This cert do not have to be valid, it is just the last cert to be
+        # This cert do not have to be valid, it is just the last cert to be
         # validated.
         self._last_cert_verified = None
         self._last_validated_cert = None
@@ -1195,7 +1255,7 @@ class CertHandler(object):
             self._cert_info = None
             self._generate_cert_func_active = False
             if generate_cert_info is not None and len(self._cert_str) > 0 and \
-                    len(self._key_str) > 0 and tmp_key_file is not \
+                            len(self._key_str) > 0 and tmp_key_file is not \
                     None and tmp_cert_file is not None:
                 self._generate_cert = True
                 self._cert_info = generate_cert_info
@@ -1214,7 +1274,7 @@ class CertHandler(object):
                     cert_str, self._cert_str, self._key_str)
             else:
                 valid, mess = self._osw.verify(self._cert_str, cert_str)
-                logger.info("CertHandler.verify_cert: %s" % mess)
+                logger.info("CertHandler.verify_cert: %s", mess)
                 return valid
         return True
 
@@ -1225,7 +1285,7 @@ class CertHandler(object):
         if (self._generate_cert and active) or client_crt is not None:
             if client_crt is not None:
                 self._tmp_cert_str = client_crt
-                #No private key for signing
+                # No private key for signing
                 self._tmp_key_str = ""
             elif self._cert_handler_extra_class is not None and \
                     self._cert_handler_extra_class.use_generate_cert_func():
@@ -1233,9 +1293,9 @@ class CertHandler(object):
                     self._cert_handler_extra_class.generate_cert(
                         self._cert_info, self._cert_str, self._key_str)
             else:
-                self._tmp_cert_str, self._tmp_key_str = self._osw\
+                self._tmp_cert_str, self._tmp_key_str = self._osw \
                     .create_certificate(
-                        self._cert_info, request=True)
+                    self._cert_info, request=True)
                 self._tmp_cert_str = self._osw.create_cert_signed_certificate(
                     self._cert_str, self._key_str, self._tmp_cert_str)
                 valid, mess = self._osw.verify(self._cert_str,
@@ -1262,12 +1322,18 @@ class SecurityContext(object):
                  debug=False, template="", encrypt_key_type="des-192",
                  only_use_keys_in_metadata=False, cert_handler_extra_class=None,
                  generate_cert_info=None, tmp_cert_file=None,
-                 tmp_key_file=None, validate_certificate=None, enc_key_files=None, enc_key_type="pem",
-                 encryption_keypairs=None, enc_cert_type="pem"):
+                 tmp_key_file=None, validate_certificate=None,
+                 enc_key_files=None, enc_key_type="pem",
+                 encryption_keypairs=None, enc_cert_type="pem",
+                 sec_backend=None):
 
         self.crypto = crypto
         assert (isinstance(self.crypto, CryptoBackend))
 
+        if sec_backend:
+            assert (isinstance(sec_backend, RSACrypto))
+        self.sec_backend = sec_backend
+
         # Your private key for signing
         self.key_file = key_file
         self.key_type = key_type
@@ -1284,8 +1350,6 @@ class SecurityContext(object):
         self.encryption_keypairs = encryption_keypairs
         self.enc_cert_type = enc_cert_type
 
-
-
         self.my_cert = read_cert_from_file(cert_file, cert_type)
 
         self.cert_handler = CertHandler(self, cert_file, cert_type, key_file,
@@ -1301,7 +1365,7 @@ class SecurityContext(object):
 
         if not template:
             this_dir, this_filename = os.path.split(__file__)
-            self.template = os.path.join(this_dir, "xml", "template.xml")
+            self.template = os.path.join(this_dir, "xml_template", "template.xml")
         else:
             self.template = template
 
@@ -1346,7 +1410,8 @@ class SecurityContext(object):
         :param key_type: The type of session key to use.
         :return: The encrypted text
         """
-        raise NotImplemented()
+        return self.crypto.encrypt_assertion(statement, enc_key, template,
+                                             key_type, node_xpath)
 
     def decrypt_keys(self, enctext, keys=None):
         """ Decrypting an encrypted text by the use of a private key.
@@ -1385,9 +1450,9 @@ class SecurityContext(object):
                 if _enctext is not None and len(_enctext) > 0:
                     return _enctext
         if key_file is not None and len(key_file.strip()) > 0:
-                _enctext = self.crypto.decrypt(enctext, key_file)
-                if _enctext is not None and len(_enctext) > 0:
-                    return _enctext
+            _enctext = self.crypto.decrypt(enctext, key_file)
+            if _enctext is not None and len(_enctext) > 0:
+                return _enctext
         return enctext
 
     def verify_signature(self, signedtext, cert_file=None, cert_type="pem",
@@ -1419,7 +1484,7 @@ class SecurityContext(object):
     def _check_signature(self, decoded_xml, item, node_name=NODE_NAME,
                          origdoc=None, id_attr="", must=False,
                          only_valid_cert=False, issuer=None):
-        #print(item)
+        # print(item)
         try:
             _issuer = item.issuer.text.strip()
         except AttributeError:
@@ -1450,51 +1515,36 @@ class SecurityContext(object):
         if not certs and not self.only_use_keys_in_metadata:
             logger.debug("==== Certs from instance ====")
             certs = [make_temp(pem_format(cert), suffix=".pem",
-                               decode=False, delete=self._xmlsec_delete_tmpfiles)
-                    for cert in cert_from_instance(item)]
+                               decode=False,
+                               delete=self._xmlsec_delete_tmpfiles)
+                     for cert in cert_from_instance(item)]
         else:
-            logger.debug("==== Certs from metadata ==== %s: %s ====" % (issuer,
-                                                                        certs))
+            logger.debug("==== Certs from metadata ==== %s: %s ====", issuer,
+                         certs)
 
         if not certs:
             raise MissingKey("%s" % issuer)
 
-        #print(certs)
+        # print(certs)
 
         verified = False
         last_pem_file = None
         for _, pem_file in certs:
             try:
                 last_pem_file = pem_file
-                if origdoc is not None:
-                    try:
-                        if self.verify_signature(origdoc, pem_file,
-                                                 node_name=node_name,
-                                                 node_id=item.id,
-                                                 id_attr=id_attr):
-                            verified = True
-                            break
-                    except Exception:
-                        if self.verify_signature(decoded_xml, pem_file,
-                                                 node_name=node_name,
-                                                 node_id=item.id,
-                                                 id_attr=id_attr):
-                            verified = True
-                            break
-                else:
-                    if self.verify_signature(decoded_xml, pem_file,
-                                             node_name=node_name,
-                                             node_id=item.id, id_attr=id_attr):
-                        verified = True
-                        break
+                if self.verify_signature(decoded_xml, pem_file,
+                                         node_name=node_name,
+                                         node_id=item.id, id_attr=id_attr):
+                    verified = True
+                    break
             except XmlsecError as exc:
-                logger.error("check_sig: %s" % exc)
+                logger.error("check_sig: %s", exc)
                 pass
             except SignatureError as exc:
-                logger.error("check_sig: %s" % exc)
+                logger.error("check_sig: %s", exc)
                 pass
             except Exception as exc:
-                logger.error("check_sig: %s" % exc)
+                logger.error("check_sig: %s", exc)
                 raise
 
         if (not verified) and (not only_valid_cert):
@@ -1543,7 +1593,8 @@ class SecurityContext(object):
 
         if not msg.signature:
             if must:
-                raise SignatureError("Required signature missing on %s" % msgtype)
+                raise SignatureError(
+                    "Required signature missing on %s" % msgtype)
             else:
                 return msg
 
@@ -1678,34 +1729,19 @@ class SecurityContext(object):
             raise TypeError("Not a Response")
 
         if response.signature:
-            self._check_signature(decoded_xml, response, class_name(response),
-                                  origdoc)
+            if "do_not_verify" in kwargs:
+                pass
+            else:
+                self._check_signature(decoded_xml, response,
+                                      class_name(response), origdoc)
         elif require_response_signature:
             raise SignatureError("Signature missing for response")
 
-        # if isinstance(response, Response) and response.assertion:
-        #     # Try to find the signing cert in the assertion
-        #     for assertion in response.assertion:
-        #         if not hasattr(assertion, 'signature') or not assertion.signature:
-        #             logger.debug("unsigned")
-        #             if must:
-        #                 raise SignatureError("Signature missing for assertion")
-        #             continue
-        #         else:
-        #             logger.debug("signed")
-        #
-        #         try:
-        #             self._check_signature(decoded_xml, assertion,
-        #                                   class_name(assertion), origdoc)
-        #         except Exception as exc:
-        #             logger.error("correctly_signed_response: %s" % exc)
-        #             raise
-
         return response
 
-    #--------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     # SIGNATURE PART
-    #--------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def sign_statement_using_xmlsec(self, statement, **kwargs):
         """ Deprecated function. See sign_statement(). """
         return self.sign_statement(statement, **kwargs)
@@ -1766,7 +1802,8 @@ class SecurityContext(object):
         return self.sign_statement(statement, class_name(
             samlp.AttributeQuery()), **kwargs)
 
-    def multiple_signatures(self, statement, to_sign, key=None, key_file=None):
+    def multiple_signatures(self, statement, to_sign, key=None, key_file=None,
+                            sign_alg=None, digest_alg=None):
         """
         Sign multiple parts of a statement
 
@@ -1785,7 +1822,9 @@ class SecurityContext(object):
                     sid = item.id
 
             if not item.signature:
-                item.signature = pre_signature_part(sid, self.cert_file)
+                item.signature = pre_signature_part(sid, self.cert_file,
+                                                    sign_alg=sign_alg,
+                                                    digest_alg=digest_alg)
 
             statement = self.sign_statement(statement, class_name(item),
                                             key=key, key_file=key_file,
@@ -1811,9 +1850,9 @@ def pre_signature_part(ident, public_key=None, identifier=None,
     """
 
     if not digest_alg:
-        digest_alg=ds.digest_default
+        digest_alg = ds.DefaultSignature().get_digest_alg()
     if not sign_alg:
-        sign_alg=ds.sig_default
+        sign_alg = ds.DefaultSignature().get_sign_alg()
     signature_method = ds.SignatureMethod(algorithm=sign_alg)
     canonicalization_method = ds.CanonicalizationMethod(
         algorithm=ds.ALG_EXC_C14N)
@@ -1923,12 +1962,14 @@ def pre_encrypt_assertion(response):
     return response
 
 
-def response_factory(sign=False, encrypt=False, **kwargs):
+def response_factory(sign=False, encrypt=False, sign_alg=None, digest_alg=None,
+                     **kwargs):
     response = samlp.Response(id=sid(), version=VERSION,
                               issue_instant=instant())
 
     if sign:
-        response.signature = pre_signature_part(kwargs["id"])
+        response.signature = pre_signature_part(kwargs["id"], sign_alg=sign_alg,
+                                                digest_alg=digest_alg)
     if encrypt:
         pass
 
@@ -1937,6 +1978,7 @@ def response_factory(sign=False, encrypt=False, **kwargs):
 
     return response
 
+
 # ----------------------------------------------------------------------------
 if __name__ == '__main__':
     import argparse
