@@ -1,4 +1,7 @@
 # coding=utf-8
+import pytest
+
+from saml2.argtree import add_path
 from saml2.authn_context import pword
 from saml2.mdie import to_dict
 from saml2 import md, assertion
@@ -50,6 +53,7 @@ mail = to_dict(md.RequestedAttribute(name="urn:oid:0.9.2342.19200300.100.1.3",
                                      friendly_name="mail",
                                      name_format=NAME_FORMAT_URI), ONTS)
 
+
 # ---------------------------------------------------------------------------
 
 
@@ -60,7 +64,7 @@ def test_filter_on_attributes_0():
     required = [a]
     ava = {"serialNumber": ["12345"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
@@ -72,10 +76,71 @@ def test_filter_on_attributes_1():
     required = [a]
     ava = {"serialNumber": ["12345"], "givenName": ["Lars"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
+
+def test_filter_on_attributes_2():
+    
+    a = to_dict(Attribute(friendly_name="surName",name="urn:oid:2.5.4.4",
+                          name_format=NAME_FORMAT_URI), ONTS)
+    required = [a]
+    ava = {"sn":["kakavas"]}
+
+    ava = filter_on_attributes(ava,required,acs=ac_factory())
+    assert list(ava.keys()) == ['sn']
+    assert ava["sn"] == ["kakavas"]
+
+
+def test_filter_on_attributes_without_friendly_name():
+    ava = {"eduPersonTargetedID": "test@example.com",
+           "eduPersonAffiliation": "test",
+           "extra": "foo"}
+    eptid = to_dict(
+        Attribute(name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
+                  name_format=NAME_FORMAT_URI), ONTS)
+    ep_affiliation = to_dict(
+        Attribute(name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1",
+                  name_format=NAME_FORMAT_URI), ONTS)
+
+    restricted_ava = filter_on_attributes(ava, required=[eptid],
+                                          optional=[ep_affiliation],
+                                          acs=ac_factory())
+    assert restricted_ava == {"eduPersonTargetedID": "test@example.com",
+                              "eduPersonAffiliation": "test"}
+
+
+def test_filter_on_attributes_with_missing_required_attribute():
+    ava = {"extra": "foo"}
+    eptid = to_dict(Attribute(
+        friendly_name="eduPersonTargetedID",
+        name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
+        name_format=NAME_FORMAT_URI), ONTS)
+    with pytest.raises(MissingValue):
+        filter_on_attributes(ava, required=[eptid], acs=ac_factory())
+
+
+def test_filter_on_attributes_with_missing_optional_attribute():
+    ava = {"extra": "foo"}
+    eptid = to_dict(Attribute(
+        friendly_name="eduPersonTargetedID",
+        name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
+        name_format=NAME_FORMAT_URI), ONTS)
+    assert filter_on_attributes(ava, optional=[eptid], acs=ac_factory()) == {}
+
+
+def test_filter_on_attributes_with_missing_name_format():
+    ava = {"eduPersonTargetedID": "test@example.com",
+           "eduPersonAffiliation": "test",
+           "extra": "foo"}
+    eptid = to_dict(Attribute(friendly_name="eduPersonTargetedID",
+                              name="urn:myown:eptid",
+                              name_format=''), ONTS)
+    ava = filter_on_attributes(ava, optional=[eptid], acs=ac_factory())
+    assert ava['eduPersonTargetedID'] == "test@example.com"
+
+
 # ----------------------------------------------------------------------
 
 def test_lifetime_1():
@@ -94,6 +159,7 @@ def test_lifetime_1():
         }}
 
     r = Policy(conf)
+
     assert r is not None
 
     assert r.get_lifetime("urn:mace:umu.se:saml:roland:sp") == {"minutes": 5}
@@ -161,25 +227,22 @@ def test_ava_filter_2():
             "lifetime": {"minutes": 5},
             "attribute_restrictions": {
                 "givenName": None,
-                "surName": None,
+                "sn": None,
                 "mail": [".*@.*\.umu\.se"],
             }
         }}
 
     policy = Policy(conf)
 
-    ava = {"givenName": "Derek",
-           "surName": "Jeter",
-           "mail": "derek@example.com"}
+    ava = {"givenName": "Derek", "sn": "Jeter", "mail": "derek@example.com"}
 
     # mail removed because it doesn't match the regular expression
-    _ava =policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None, [mail],
-                        [gn, sn])
+    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None, [mail],
+                         [gn, sn])
 
-    assert _eq(sorted(list(_ava.keys())), ["givenName", "surName"])
+    assert _eq(sorted(list(_ava.keys())), ["givenName", 'sn'])
 
-    ava = {"givenName": "Derek",
-           "surName": "Jeter"}
+    ava = {"givenName": "Derek", "sn": "Jeter"}
 
     # it wasn't there to begin with
     try:
@@ -214,7 +277,7 @@ def test_ava_filter_dont_fail():
 
     # mail removed because it doesn't match the regular expression
     # So it should fail if the 'fail_on_ ...' flag wasn't set
-    _ava = policy.filter(ava,'urn:mace:umu.se:saml:roland:sp', None,
+    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None,
                          [mail], [gn, sn])
 
     assert _ava
@@ -228,6 +291,7 @@ def test_ava_filter_dont_fail():
 
     assert _ava
 
+
 def test_filter_attribute_value_assertions_0(AVA):
     p = Policy({
         "default": {
@@ -377,7 +441,7 @@ def test_filter_values_req_2():
     required = [a1, a2]
     ava = {"serialNumber": ["12345"], "givenName": ["Lars"]}
 
-    raises(MissingValue, filter_on_attributes, ava, required)
+    raises(MissingValue, filter_on_attributes, ava, required, acs=ac_factory())
 
 
 def test_filter_values_req_3():
@@ -389,7 +453,7 @@ def test_filter_values_req_3():
     required = [a]
     ava = {"serialNumber": ["12345"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
@@ -403,7 +467,7 @@ def test_filter_values_req_4():
     required = [a]
     ava = {"serialNumber": ["12345"]}
 
-    raises(MissingValue, filter_on_attributes, ava, required)
+    raises(MissingValue, filter_on_attributes, ava, required, acs=ac_factory())
 
 
 def test_filter_values_req_5():
@@ -415,7 +479,7 @@ def test_filter_values_req_5():
     required = [a]
     ava = {"serialNumber": ["12345", "54321"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
@@ -429,7 +493,7 @@ def test_filter_values_req_6():
     required = [a]
     ava = {"serialNumber": ["12345", "54321"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["54321"]
 
@@ -446,7 +510,7 @@ def test_filter_values_req_opt_0():
 
     ava = {"serialNumber": ["12345", "54321"]}
 
-    ava = filter_on_attributes(ava, [r], [o])
+    ava = filter_on_attributes(ava, [r], [o], acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert _eq(ava["serialNumber"], ["12345", "54321"])
 
@@ -464,7 +528,7 @@ def test_filter_values_req_opt_1():
 
     ava = {"serialNumber": ["12345", "54321"]}
 
-    ava = filter_on_attributes(ava, [r], [o])
+    ava = filter_on_attributes(ava, [r], [o], acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert _eq(ava["serialNumber"], ["12345", "54321"])
 
@@ -500,7 +564,7 @@ def test_filter_values_req_opt_2():
     ava = {"surname": ["Hedberg"], "givenName": ["Roland"],
            "eduPersonAffiliation": ["staff"], "uid": ["rohe0002"]}
 
-    raises(MissingValue, "filter_on_attributes(ava, r, o)")
+    raises(MissingValue, "filter_on_attributes(ava, r, o, acs=ac_factory())")
 
 
 # ---------------------------------------------------------------------------
@@ -691,7 +755,7 @@ def test_req_opt():
                 is_required="false"), ONTS)]
 
     policy = Policy()
-    ava = {'givenname': 'Roland', 'surname': 'Hedberg',
+    ava = {'givenname': 'Roland', 'sn': 'Hedberg',
            'uid': 'rohe0002', 'edupersonaffiliation': 'staff'}
 
     sp_entity_id = "urn:mace:example.com:saml:curt:sp"
@@ -768,10 +832,21 @@ def test_assertion_with_noop_attribute_conv():
     })
     name_id = NameID(format=NAMEID_FORMAT_TRANSIENT, text="foobar")
     issuer = Issuer(text="entityid", format=NAMEID_FORMAT_ENTITY)
-    msg = ast.construct("sp_entity_id", "in_response_to", "consumer_url",
-                        name_id, [AttributeConverterNOOP(NAME_FORMAT_URI)],
-                        policy, issuer=issuer, authn_decl=ACD,
-                        authn_auth="authn_authn")
+
+    farg = add_path(
+        {},
+        ['subject', 'subject_confirmation', 'method', saml.SCM_BEARER])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'in_response_to', 'in_response_to'])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'recipient', 'consumer_url'])
+
+    msg = ast.construct(
+        "sp_entity_id", [AttributeConverterNOOP(NAME_FORMAT_URI)], policy,
+        issuer=issuer, farg=farg, authn_decl=ACD, name_id=name_id,
+        authn_auth="authn_authn")
 
     print(msg)
     for attr in msg.attribute_statement[0].attribute:
@@ -784,7 +859,7 @@ def test_assertion_with_noop_attribute_conv():
 
 
 # THis test doesn't work without a MetadataStore instance
-#def test_filter_ava_5():
+# def test_filter_ava_5():
 #    policy = Policy({
 #        "default": {
 #            "lifetime": {"minutes": 15},
@@ -816,10 +891,20 @@ def test_assertion_with_zero_attributes():
     })
     name_id = NameID(format=NAMEID_FORMAT_TRANSIENT, text="foobar")
     issuer = Issuer(text="entityid", format=NAMEID_FORMAT_ENTITY)
-    msg = ast.construct("sp_entity_id", "in_response_to", "consumer_url",
-                        name_id, [AttributeConverterNOOP(NAME_FORMAT_URI)],
-                        policy, issuer=issuer, authn_decl=ACD,
-                        authn_auth="authn_authn")
+    farg = add_path(
+        {},
+        ['subject', 'subject_confirmation', 'method', saml.SCM_BEARER])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'in_response_to', 'in_response_to'])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'recipient', 'consumer_url'])
+
+    msg = ast.construct(
+        "sp_entity_id", [AttributeConverterNOOP(NAME_FORMAT_URI)], policy,
+        issuer=issuer, authn_decl=ACD, authn_auth="authn_authn",
+        name_id=name_id, farg=farg)
 
     print(msg)
     assert msg.attribute_statement == []
@@ -837,11 +922,21 @@ def test_assertion_with_authn_instant():
     })
     name_id = NameID(format=NAMEID_FORMAT_TRANSIENT, text="foobar")
     issuer = Issuer(text="entityid", format=NAMEID_FORMAT_ENTITY)
-    msg = ast.construct("sp_entity_id", "in_response_to", "consumer_url",
-                        name_id, [AttributeConverterNOOP(NAME_FORMAT_URI)],
-                        policy, issuer=issuer, authn_decl=ACD,
-                        authn_auth="authn_authn",
-                        authn_instant=1234567890)
+
+    farg = add_path(
+        {},
+        ['subject', 'subject_confirmation', 'method', saml.SCM_BEARER])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'in_response_to', 'in_response_to'])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'recipient', 'consumer_url'])
+
+    msg = ast.construct(
+        "sp_entity_id", [AttributeConverterNOOP(NAME_FORMAT_URI)], policy,
+        issuer=issuer, authn_decl=ACD, authn_auth="authn_authn",
+        authn_instant=1234567890, name_id=name_id, farg=farg)
 
     print(msg)
     assert msg.authn_statement[0].authn_instant == "2009-02-13T23:31:30Z"
@@ -849,3 +944,4 @@ def test_assertion_with_authn_instant():
 
 if __name__ == "__main__":
     test_assertion_2()
+
