@@ -1,31 +1,12 @@
 from contextlib import closing
-from saml2 import saml, sigver
-from saml2 import md
+from saml2 import sigver
 from saml2 import config
 from saml2.assertion import Policy
 from saml2.attribute_converter import ac_factory
-from saml2.extension import mdui
-from saml2.extension import idpdisc
-from saml2.extension import dri
-from saml2.extension import mdattr
-from saml2.extension import ui
 from pathutils import full_path
 from saml2.mdstore import MetadataStore
 from saml2.server import Server
-from saml2 import xmldsig
-from saml2 import xmlenc
 
-ONTS = {
-    saml.NAMESPACE: saml,
-    mdui.NAMESPACE: mdui,
-    mdattr.NAMESPACE: mdattr,
-    dri.NAMESPACE: dri,
-    ui.NAMESPACE: ui,
-    idpdisc.NAMESPACE: idpdisc,
-    md.NAMESPACE: md,
-    xmldsig.NAMESPACE: xmldsig,
-    xmlenc.NAMESPACE: xmlenc
-}
 
 ATTRCONV = ac_factory(full_path("attributemaps"))
 sec_config = config.Config()
@@ -33,7 +14,7 @@ sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])
 
 __author__ = 'rolandh'
 
-MDS = MetadataStore(ONTS.values(), ATTRCONV, sec_config,
+MDS = MetadataStore(ATTRCONV, sec_config,
                     disable_ssl_certificate_validation=True)
 MDS.imp([{"class": "saml2.mdstore.MetaDataMD",
           "metadata": [(full_path("swamid.md"),)]}])
@@ -90,7 +71,7 @@ def test_filter_ava3():
         }
     })
 
-    mds = MetadataStore(list(ONTS.values()), ATTRCONV, sec_config,
+    mds = MetadataStore(ATTRCONV, sec_config,
                         disable_ssl_certificate_validation=True)
     mds.imp([{"class": "saml2.mdstore.MetaDataFile",
               "metadata": [(full_path("entity_cat_sfs_hei.xml"),)]}])
@@ -114,7 +95,7 @@ def test_filter_ava4():
         }
     })
 
-    mds = MetadataStore(list(ONTS.values()), ATTRCONV, sec_config,
+    mds = MetadataStore(ATTRCONV, sec_config,
                         disable_ssl_certificate_validation=True)
     mds.imp([{"class": "saml2.mdstore.MetaDataFile",
               "metadata": [(full_path("entity_cat_re_nren.xml"),)]}])
@@ -140,7 +121,7 @@ def test_filter_ava5():
         }
     })
 
-    mds = MetadataStore(list(ONTS.values()), ATTRCONV, sec_config,
+    mds = MetadataStore(ATTRCONV, sec_config,
                         disable_ssl_certificate_validation=True)
     mds.imp([{"class": "saml2.mdstore.MetaDataFile",
               "metadata": [(full_path("entity_cat_re.xml"),)]}])
